from __future__ import unicode_literals

import logging

import requests

try:
    from urlparse import urljoin
except ImportError:
    from urllib.parse import urljoin

from .exceptions import SendyError


log = logging.getLogger(__name__)


def convert_to_utf8(value):
    try:
        unicode
    except NameError:
        return value
    if isinstance(value, str):
        value = unicode(value)
    if isinstance(value, unicode):
        return value.decode('utf-8')
    return value


def bind_api(**config):
    class APIMethod:
        path = config['path']
        allowed_param = config.get('allowed_param', [])
        method = config.get('method', 'GET')
        require_auth = config.get('require_auth', True)
        success_status_code = config.get('success_status_code', 200)
        success_message = config.get('success_message', None)
        extra_param = config.get('extra_param', None)
        fail_silently = config.get('fail_silently', True)

        def __init__(self, api, args, kwargs):
            self.api = api
            self.headers = kwargs.pop('headers', {})
            self.post_data = kwargs.pop('post_data', None)
            self.build_parameters(args, kwargs)
            self.path = self.path.format(**self.parameters)  # Sub any URL vars
            self.host = api.host

            if not isinstance(self.success_message, (list, tuple)):
                self.success_message = [self.success_message]

        def build_parameters(self, args, kwargs):
            self.parameters = {}
            for idx, arg in enumerate(args):
                if arg is None:
                    continue

                try:
                    self.parameters[self.allowed_param[idx]] = convert_to_utf8(arg)
                except IndexError:
                    raise ValueError('Too many parameters supplied!')

            for k, arg in kwargs.items():
                if arg is None:
                    continue
                if k in self.parameters:
                    raise ValueError(
                        'Multiple values for parameter {0} supplied!'.format(k)
                    )

                self.parameters[k] = convert_to_utf8(arg)

            if self.extra_param is not None:
                self.parameters.update(self.extra_param)

            if self.require_auth and self.api.api_key is not None:
                self.parameters['api_key'] = self.api.api_key

        def execute(self):
            # Build the request URL
            url = urljoin(self.host, self.path)

            if self.api.debug:
                log.info(
                    '* Sending {0}: {1}\nHeaders: {2}\nData:{3}'.format(
                        self.method, url, self.headers, self.post_data
                    )
                )

            response = requests.request(
                self.method, url, headers=self.headers, data=self.parameters
            )
            data = response.content
            if callable(self.success_message):
                try:
                    return self.success_message(data)
                except ValueError:
                    if not self.fail_silently:
                        raise SendyError(data)

            success = data in self.success_message
            if not success and not self.fail_silently:
                raise SendyError(data)
            return data

    def _call(api, *args, **kwargs):
        method = APIMethod(api, args, kwargs)
        return method.execute()

    return _call



from __future__ import unicode_literals

import logging

from .binder import bind_api

log = logging.getLogger(__name__)


class SendyAPI:
    """ Class used to map to all Sendy API endpoints
    """

    def __init__(self, host, api_key=None, debug=False):
        self.host = host
        if self.host[-1] != '/':
            self.host = '{0}/'.format(self.host)
        self.api_key = api_key
        self.debug = debug

    subscribe = bind_api(
        path='subscribe',
        allowed_param=[
            'list',
            'email',
            'name',
        ],
        extra_param={'boolean': 'true'},
        success_message='1',
        require_auth=False,
        method='POST',
    )

    unsubscribe = bind_api(
        path='unsubscribe',
        allowed_param=['list', 'email'],
        extra_param={'boolean': 'true'},
        success_message='1',
        require_auth=False,
        method='POST',
    )

    delete = bind_api(
        path='api/subscribers/delete.php',
        allowed_param=['list_id', 'email'],
        success_message='1',
        method='POST',
    )

    subscription_status = bind_api(
        path='api/subscribers/subscription-status.php',
        allowed_param=['list_id', 'email'],
        success_message=[
            'Subscribed',
            'Unsubscribed',
            'Unconfirmed',
            'Bounced',
            'Soft bounced',
            'Complained',
        ],
        method='POST',
    )

    subscriber_count = bind_api(
        path='api/subscribers/active-subscriber-count.php',
        allowed_param=['list_id'],
        success_message=int,
        method='POST',
    )

    create_campaign = bind_api(
        path='api/campaigns/create.php',
        allowed_param=[
            'from_name',
            'from_email',
            'reply_to',
            'title',
            'subject',
            'plain_text',
            'html_text',
            'list_ids',
            'brand_id',
            'query_string',
            'send_campaign',
        ],
        success_message=[
            'Campaign created', 'Campaign created and now sending'
        ],
        method='POST',
    )



from __future__ import unicode_literals

name = 'sendypy'

VERSION = (0, 1, 5, 'beta', 0)


def get_version():
    'Returns a PEP 386-compliant version number from VERSION.'
    assert len(VERSION) == 5
    assert VERSION[3] in ('alpha', 'beta', 'rc', 'final')

    # Now build the two parts of the version number:
    # main = X.Y[.Z]
    # sub = .devN - for pre-alpha releases
    #     | {a|b|c}N - for alpha, beta and rc releases

    parts = 2 if VERSION[2] == 0 else 3
    main = '.'.join(str(x) for x in VERSION[:parts])

    sub = ''
    if VERSION[3] != 'final':
        mapping = {'alpha': 'a', 'beta': 'b', 'rc': 'c'}
        sub = mapping[VERSION[3]] + str(VERSION[4])

    return str(main + sub)



import os
from setuptools import setup, find_packages


if os.path.exists('README.rst'):
    long_description = open('README.rst', 'r').read()
else:
    long_description = 'See https://bitbucket.org/netlandish/sendypy/'


setup(
    name='sendypy',
    version=__import__('sendy').get_version(),
    packages=find_packages(),
    description='Python Interface for the Sendy API',
    author='Netlandish Inc.',
    author_email='hello@netlandish.com',
    url='https://bitbucket.org/netlandish/sendypy/',
    long_description=long_description,
    platforms=['any'],
    install_requires=['requests>=2.19.1'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Environment :: Web Environment',
    ],
    include_package_data=True,
)
